@@ -18,6 +18,7 @@ from ugrid.c_structures import (
     CUGridNetwork1D,
     decode_byte_vector_to_list_of_strings,
     decode_byte_vector_to_string,
+    numpy_array_to_ctypes,
 )
 from ugrid.errors import UGridError
 from ugrid.py_structures import UGridContacts, UGridMesh1D, UGridMesh2D, UGridNetwork1D
@@ -1159,3 +1160,148 @@ class UGrid:
         )
 
         return data_vec
+
+    def __variable_int_define(self, variable_name: str):
+        """Defines a new integer variable
+
+        Args:
+            variable_name (str): The variable name.
+        """
+
+        variable_name_long = self.__adjust_name(variable_name)
+        c_variable_name_encoded = c_char_p(variable_name_long.encode("ASCII"))
+
+        self.__execute_function(
+            self.lib.ug_variable_int_define, self._file_id, c_variable_name_encoded
+        )
+
+    def __attribute_define(
+        self, variable_name: str, attribute_name: str, attribute_values
+    ):
+        """Defines a new variable attribute with a name and an array of values.
+
+        Args:
+            variable_name (str): The variable name.
+            attribute_name (str): The attribute name.
+            attribute_values: The attribute values. Can be a numpy array of int, floats or a string
+        """
+
+        variable_name_long = self.__adjust_name(variable_name)
+        c_variable_name_encoded = c_char_p(variable_name_long.encode("ASCII"))
+
+        attribute_name_long = self.__adjust_name(attribute_name)
+        c_attribute_name_encoded = c_char_p(attribute_name_long.encode("ASCII"))
+
+        attribute_values_len = c_int(len(attribute_values))
+
+        if isinstance(
+            attribute_values, np.ndarray
+        ) and attribute_values.dtype is np.dtype("int32"):
+            fun = self.lib.ug_attribute_int_define
+            c_attribute_values = numpy_array_to_ctypes(attribute_values)
+        elif isinstance(
+            attribute_values, np.ndarray
+        ) and attribute_values.dtype is np.dtype("float"):
+            fun = self.lib.ug_attribute_double_define
+            c_attribute_values = numpy_array_to_ctypes(attribute_values)
+        elif isinstance(attribute_values, str):
+            fun = self.lib.ug_attribute_char_define
+            c_attribute_values = c_char_p(attribute_values.encode("ASCII"))
+        else:
+            raise UGridError(
+                "__attribute_define: attribute_values types are not supported"
+            )
+
+        self.__execute_function(
+            fun,
+            self._file_id,
+            c_variable_name_encoded,
+            c_attribute_name_encoded,
+            c_attribute_values,
+            attribute_values_len,
+        )
+
+    def variable_int_with_attributes_define(
+        self, variable_name: str, variable_dict: dict
+    ):
+        """Defines a new variable with attributes. It can be used to write the coordinate reference system to file.
+
+        Args:
+            variable_name (str): The new variable name.
+            variable_dict (dict): A dictionary containing the variable values and names
+        """
+
+        self.__variable_int_define(variable_name)
+        for names, values in variable_dict.items():
+            self.__attribute_define(variable_name, names, values)
+
+    def attribute_global_define(self, variable_dict: dict):
+        """Defines new global attributes
+
+        Args:
+            variable_dict (dict): A dictionary containing the attribute names and values.
+        """
+        for attribute_name, attribute_value in variable_dict.items():
+
+            attribute_name_long = self.__adjust_name(attribute_name)
+            c_attribute_name_encoded = c_char_p(attribute_name_long.encode("ASCII"))
+
+            c_attribute_value_encoded = c_char_p(attribute_value.encode("ASCII"))
+            attribute_values_len = c_int(len(attribute_value))
+
+            self.__execute_function(
+                self.lib.ug_attribute_global_char_define,
+                self._file_id,
+                c_attribute_name_encoded,
+                c_attribute_value_encoded,
+                attribute_values_len,
+            )
+
+    def network1d_get_attributes(self, name: str):
+        """Get a dictionary of network1d default attribute names and the corresponding default values.
+
+        Args:
+            name (str): The network1d name.
+
+        Returns:
+            dict: A dictionary containing the attribute names and the corresponding default values.
+        """
+
+        return {
+            "cf_role": "mesh_topology",
+            "long_name": "Topology data of 1D network",
+            "topology_dimension": 1,
+            "node_dimension": f"{name}_nNodes",
+            "node_coordinates": f"{name}_node_x {name}_node_y",
+            "node_id": f"{name}_node_id",
+            "node_long_name": f"{name}_node_long_name",
+            "edge_dimension": f"{name}_nEdges",
+            "edge_node_connectivity": f"{name}_edge_nodes",
+            "edge_length": f"{name}_edge_length",
+            "edge_id": f"{name}_edge_id",
+            "edge_long_name": f"{name}_edge_long_name",
+            "edge_geometry": f"{name}_edge_geometry",
+        }
+
+    def mesh2d_get_attributes(self, name: str):
+        """Get a dictionary of mesh2d default attribute names.
+
+        Args:
+            name (str): The mesh2d name.
+
+        Returns:
+            dict: A dictionary containing the attribute names and the corresponding default values.
+        """
+        return {
+            "cf_role": "mesh_topology",
+            "long_name": "Topology data of 2D mesh",
+            "topology_dimension": 2,
+            "node_dimension": f"{name}_nNodes",
+            "node_coordinates": f"{name}_node_x {name}_node_y",
+            "edge_dimension": f"{name}_nNodes",
+            "edge_node_connectivity": f"{name}_edge_nodes",
+            "face_dimension": f"{name}_nFaces",
+            "face_node_connectivity": f"{name}_face_nodes",
+            "max_face_nodes_dimension": f"{name}_nMax_face_nodes",
+            "face_coordinates": f"{name}_face_x {name}_face_y",
+        }
